Package: sftrack
Title: Modern Classes for Tracking and Movement Data
Version: 0.5.5
Date: 2026-05-17
Authors@R: c(
    person(given = "Matthew",
           family = "Boone",
           role = "aut",
           email = "mttboone@gmail.com",
           comment = c(ORCID = "0000-0002-8070-4715")),
    person(given = "Rocío",
           family = "Joo",
           role = "aut",
           email = "rocio.joo@globalfishingwatch.org",
           comment = c(ORCID = "0000-0003-0319-4210")),
    person(given = "Mathieu",
           family = "Basille",
           role = c("aut", "cre"),
           email = "mathieu@basille.org",
           comment = c(ORCID = "0000-0001-9366-7127")))
Depends: R (>= 3.2.0)
Imports: sf
Description: Modern classes for tracking and movement data, building
    on 'sf' spatial infrastructure, and early theoretical work from
    Turchin (1998, ISBN: 9780878938476), and Calenge et al. (2009)
    <doi:10.1016/j.ecoinf.2008.10.002>. Tracking data are series of
    locations with at least 2-dimensional spatial coordinates (x,y), a
    time index (t), and individual identification (id) of the object
    being monitored; movement data are made of trajectories, i.e. the
    line representation of the path, composed by steps (the
    straight-line segments connecting successive locations). 'sftrack'
    is designed to handle movement of both living organisms and
    inanimate objects.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr
URL: https://mablab.org/sftrack/, https://github.com/mablab/sftrack
BugReports: https://github.com/mablab/sftrack/issues
Suggests: testthat (>= 2.1.0), utils, ggplot2, adehabitatLT, knitr,
        geosphere, scales, covr, rmarkdown, lwgeom
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-18 07:00:39 UTC; mathieu
Author: Matthew Boone [aut] (ORCID: <https://orcid.org/0000-0002-8070-4715>),
  Rocío Joo [aut] (ORCID: <https://orcid.org/0000-0003-0319-4210>),
  Mathieu Basille [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9366-7127>)
Maintainer: Mathieu Basille <mathieu@basille.org>
Repository: CRAN
Date/Publication: 2026-05-18 07:30:02 UTC
Built: R 4.4.3; ; 2026-05-18 08:05:44 UTC; unix
