# Generated by roxygen2: do not edit by hand

S3method("$<-",s_group)
S3method("[",c_grouping)
S3method("[",sftrack)
S3method("[",sftraj)
S3method("[<-",c_grouping)
S3method("[<-",s_group)
S3method("[[<-",sftrack)
S3method("[[<-",sftraj)
S3method("active_group<-",c_grouping)
S3method("active_group<-",sftrack)
S3method("active_group<-",sftraj)
S3method(as.data.frame,c_grouping)
S3method(as_sftrack,data.frame)
S3method(as_sftrack,ltraj)
S3method(as_sftrack,sf)
S3method(as_sftrack,sftraj)
S3method(as_sftraj,data.frame)
S3method(as_sftraj,ltraj)
S3method(as_sftraj,sf)
S3method(as_sftraj,sftrack)
S3method(c,c_grouping)
S3method(c,s_group)
S3method(cbind,sftrack)
S3method(format,c_grouping)
S3method(format,s_group)
S3method(geom_sftrack,sftrack)
S3method(geom_sftrack,sftraj)
S3method(group_labels,c_grouping)
S3method(group_labels,sftrack)
S3method(group_labels,sftraj)
S3method(group_names,c_grouping)
S3method(group_names,sftrack)
S3method(group_names,sftraj)
S3method(plot,sftrack)
S3method(plot,sftraj)
S3method(print,c_grouping)
S3method(print,s_group)
S3method(print,sftrack)
S3method(print,sftraj)
S3method(rbind,sftrack)
S3method(rbind,sftraj)
S3method(st_coordinates,sftraj)
S3method(str,c_grouping)
S3method(summary,c_grouping)
S3method(summary,sftrack)
S3method(summary,sftraj)
export("active_group<-")
export(active_group)
export(as_sftrack)
export(as_sftraj)
export(calc_sort_index)
export(check_NA_coords)
export(check_NA_group)
export(check_group_id)
export(check_group_names)
export(check_names_exist)
export(check_ordered)
export(check_t_regular)
export(check_time)
export(coord_traj)
export(dup_timestamp)
export(fix_zero)
export(geom_sftrack)
export(group_labels)
export(group_names)
export(is_linestring)
export(make_c_grouping)
export(make_s_group)
export(make_step_geom)
export(merge_traj)
export(new_sftrack)
export(new_sftraj)
export(pts_traj)
export(step_metrics)
export(step_recalc)
export(summary_sftrack)
export(which_duplicated)
import(sf)
importFrom(graphics,lcm)
importFrom(graphics,plot)
importFrom(stats,aggregate)
importFrom(utils,str)
