% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checks.R
\name{check_time}
\alias{check_time}
\title{Check if time is integer or posix}
\usage{
check_time(time)
}
\arguments{
\item{time}{a vector of time}
}
\description{
Check if time is integer or posix
}
