% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sftrack.R
\name{new_sftrack}
\alias{new_sftrack}
\title{Define an sftrack}
\usage{
new_sftrack(data, group_col, sf_col, time_col, error_col = NA)
}
\arguments{
\item{data}{data.frame with multi_burst column, geometry column, time_col (integer/POSIXct), and error column (optional)}

\item{group_col}{column name of grouping info in `data`}

\item{sf_col}{column name of geometry info in `data`}

\item{time_col}{column name of time info in `data`}

\item{error_col}{column name of error info in `data`}
}
\description{
Define an sftrack
}
