% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc_util.R
\name{summary_sftrack}
\alias{summary_sftrack}
\title{Summarize sftrack objects}
\usage{
summary_sftrack(x)
}
\arguments{
\item{x}{an sftrack object}
}
\description{
Summarize sftrack objects
}
