% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checks.R
\name{check_group_id}
\alias{check_group_id}
\title{Check there is a grouping id present}
\usage{
check_group_id(x)
}
\arguments{
\item{x}{a c_grouping}
}
\description{
Check there is a grouping id present
}
